package ru.dbotthepony.mc.otm.triggers

import com.google.gson.JsonObject
import com.google.gson.JsonPrimitive
import net.minecraft.advancements.Criterion
import net.minecraft.advancements.critereon.*
import net.minecraft.resources.ResourceLocation
import net.minecraft.server.level.ServerPlayer
import ru.dbotthepony.mc.otm.OverdriveThatMatters
import ru.dbotthepony.mc.otm.android.AndroidResearchType
import ru.dbotthepony.mc.otm.core.set
import java.util.Optional
import java.util.function.Predicate

object AndroidResearchTrigger : SimpleCriterionTrigger<AndroidResearchTrigger.Instance>() {
	val id = ResourceLocation(OverdriveThatMatters.MOD_ID, "android_research")

	override fun createInstance(
		p_66248_: JsonObject,
		p_286603_: Optional<ContextAwarePredicate>,
		p_66250_: DeserializationContext
	): Instance {
		return Instance(
			p_66248_["research"]?.asString?.let(::ResourceLocation)
		)
	}

	fun trigger(player: ServerPlayer, research: AndroidResearchType) {
		trigger(player) {
			it.test(research)
		}
	}

	class Instance(val research: ResourceLocation?) : AbstractCriterionTriggerInstance(Optional.empty()), Predicate<AndroidResearchType> {
		constructor(research: AndroidResearchType) : this(research.id)

		override fun test(t: AndroidResearchType): Boolean {
			return research == null || t.id == research
		}

		override fun serializeToJson(): JsonObject {
			return super.serializeToJson().also {
				if (research != null)
					it["research"] = JsonPrimitive(research.toString())
			}
		}

		fun criterion() = Criterion(AndroidResearchTrigger, this)
	}
}
